import pytest

from src.kangaroo import Kangaroo


@pytest.fixture
def kangaroo() -> Kangaroo:
    """
    Create an instance to be used across the test suite
    :return: Kangaroo instance
    """
    kangaroo = Kangaroo()
    yield kangaroo


def test_send_with_success(kangaroo: Kangaroo) -> None:
    """
    Test if message is sent successfully
    """
    expected = "Hello"

    r = kangaroo.listen(3000)
    kangaroo.send(3000, expected)

    result = r.get_message()
    assert expected == result


def test_listen_with_success(kangaroo: Kangaroo) -> None:
    """
    Test if message is received successfully
    """
    expected = "World"

    r = kangaroo.listen(3001)
    kangaroo.send(3001, expected)

    assert r.has_new_message()


def test_get_message_fails(kangaroo: Kangaroo) -> None:
    """
    Test if the message is empty
    """
    r = kangaroo.listen(3003)
    assert r.get_message() == ""


def test_has_new_message_fails(kangaroo: Kangaroo) -> None:
    """
    Test if there's no new message
    """
    r = kangaroo.listen(3004)
    assert not r.has_new_message()



import socket
import threading


class Kangaroo:
    def __init__(self):
        self.__localhost = "127.0.0.1"
        self.__message = ""
        self.__has_new_message = False

    def send(self, port: int, message: str) -> None:
        """
        Send message to given port
        :param port: Port to send the message
        :param message: Message to be sent
        :return: None
        """
        socket_conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        socket_conn.connect((self.__localhost, port))
        socket_conn.sendall(bytes(message, "utf-8"))
        socket_conn.close()

    def listen(self, port: int):
        """
        Listen to a given port
        :param port: Port to send the message
        :return: Kangaroo object
        """
        socket_conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        socket_conn.bind((self.__localhost, port))
        socket_conn.listen(1)

        self.__listening_thread = threading.Thread(
            target=self.__listening, args=(socket_conn,)
        )
        self.__listening_thread.setDaemon(True)
        self.__listening_thread.start()

        return self

    def __listening(self, socket_conn: socket.socket) -> None:
        """
        Infinite loop to listen to a port
        :param socket_conn: Socket to be listen
        :return: None
        """
        while True:
            conn, _ = socket_conn.accept()
            self.__message = conn.recv(1024).decode("UTF-8")
            self.__has_new_message = True

    def has_new_message(self) -> bool:
        """
        Returns if there's a new message
        :return: True if there's a new message
        """
        return self.__has_new_message

    def get_message(self) -> str:
        """
        Returns the last message receive
        :return: Last message
        """
        if self.__has_new_message:
            self.__has_new_message = False
            return self.__message
        else:
            return ""
